from datetime import datetime

from .egypt_codes import GOVERNORATES_CODES_MAP


class NationalID:
    """National ID class, will include the skelton of it's properties
    and verification.
    """

    def __init__(self, id_number: str) -> None:
        """
        Args:
            id_number (int): Egyptian national id number
        """
        self.id_number = id_number

    def _validate(self) -> bool:
        """Validates the id number parts

        Returns:
            bool: True if it is a valid egyptian id number, False if not
        """
        if not all([self.id_number.isdigit(), len(self.id_number) == 14]):
            return False

        self.century = int(self.id_number[0])
        self.year = int(self.id_number[1:3])
        self.month = int(self.id_number[3:5])
        self.day = int(self.id_number[5:7])
        self.governorate = int(self.id_number[7:9])
        self.unique_num = int(self.id_number[9:13])
        self.verification_digit = int(self.id_number[13])

        current_datetime = datetime.now()

        century_check = self.century in [2, 3]  # we will deal with 1900 ~ 2099
        year_check = (
            (self.year <= current_datetime.year - 2000) if self.century == 3 else True
        )  # can't be in the future
        month_check = self.month in range(1, 13)

        # check for months that have 31 day and febuary which is 28 or 29
        if self.month in [1, 3, 5, 7, 8, 10, 12]:
            day_check = self.day in range(1, 32)
        elif self.month in [4, 6, 9, 11]:
            day_check = self.day in range(1, 31)
        elif self.month == 2:
            day_check = self.day in range(30) if self.year % 4 == 0 else self.day in range(29)
        else:
            day_check = False

        governorate_check = self.governorate in GOVERNORATES_CODES_MAP

        return all([century_check, year_check, month_check, day_check, governorate_check])

    def get_info(self) -> tuple:
        """Process the data from the id number into readable format

        Returns:
            tuple(bool, str): True if valid_id else False, json string for the collective information that will return to user
        """
        id_owner_data = {}
        if not self._validate():
            number_error_msg = f"Invalid national ID number: {self.id_number}. Please enter the correct one"
            return False, {"error": number_error_msg}

        id_owner_data["year_of_birth"] = f"20{self.year}" if self.century == 3 else f"19{self.year}"
        id_owner_data["month_of_birth"] = f"{self.month}"
        id_owner_data["day_of_birth"] = f"{self.day}"
        id_owner_data["governorate"] = GOVERNORATES_CODES_MAP[self.governorate]
        id_owner_data["type"] = "Male" if self.unique_num % 2 != 0 else "Female"

        return True, id_owner_data



GOVERNORATES_CODES_MAP = {
    1: "Cairo",
    2: "Alexandria",
    3: "Port Said",
    4: "Suez",
    11: "Damietta",
    12: "Al Daqhlia",
    13: "Al Sharkia",
    14: "Al Qaliobia",
    15: "Kafr Alsheekh",
    16: "Al Gharbia",
    17: "Al Monuefia",
    18: "Al Bohira",
    19: "Al Ismaellia",
    21: "Al Giza",
    22: "Beni Suief",
    23: "Al Fayoum",
    24: "Al minia",
    25: "Assyout",
    26: "Suhag",
    27: "Quena",
    28: "Aswan",
    29: "Luxor",
    31: "AlBahr AlAhmar",
    32: "AlWadi AlJadid",
    33: "Matrooh",
    34: "North Sinai",
    35: "South Sinai",
    88: "Outside Egypt",
}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lib']

package_data = \
{'': ['*']}

install_requires = \
['WebTest==2.0.35',
 'bottle==0.12.17',
 'loguru==0.5.3',
 'pytest-cov==2.10.1',
 'pytest==5.4.3',
 'requests==2.23.0']

setup_kwargs = {
    'name': 'national-id',
    'version': '1.0.1',
    'description': 'Egyptian national ID validator and data-extractor AP',
    'long_description': '# Egyptian national ID validator and data-extractor API\n\n[![Actions Status](https://github.com/waleedhammam/national_id/workflows/national_id/badge.svg?query=branch%3Amain)](https://github.com/waleedhammam/national_id/actions?query=branch%3Amain)\n[![codecov.io](https://codecov.io/github/waleedhammam/national_id/coverage.svg?branch=main)](https://codecov.io/github/waleedhammam/national_id?branch=main)\n\n## Requirments\n\n- python3.6 or later\n- pip3\n- bottle\n\n## How to run\n\n- Install requirments `pip3 install -r requirements.txt`\n- Run `python3 server.py`\n- Server will be running on port 8001\n- Endpoint can be reached at http://localhost:8001/get_info\n\n## Endpoint\n\n- `/get_info`\n  - Accepts post requests with "Content-Type: application/json" Header\n    - Example request:\n\n      ```bash\n      curl -H "Content-Type: application/json" -d \'{"id_number": "29009121201812"}\' -XPOST http://localhost:8001/get_info\n      ```\n\n  - Response\n    - 200 OK, json_info: national id is validated and info extraction ok\n    - 400 Bad Request: Wrong national id number\n    - 500 Internal Server Error: Invalid request from user (invalid json, invalid form of data)\n\n    Example response:\n\n      ```bash\n      {"nationl_id_data": {"year_of_birth": "1994", "month_of_birth": "9", "day_of_birth": "15", "governorate": "Al Daqhlia", "type": "Male"}}\n      ```\n\n## Validations and checks\n\n- According to [this source](https://ar.wikipedia.org/wiki/%D8%A8%D8%B7%D8%A7%D9%82%D8%A9_%D8%A7%D9%84%D8%B1%D9%82%D9%85_%D8%A7%D9%84%D9%82%D9%88%D9%85%D9%8A_%D8%A7%D9%84%D9%85%D8%B5%D8%B1%D9%8A%D8%A9)\n\nThe national ID consists of the following:\n\n  ```bash\n  +-+--+--+--+--+----+-+\n  |2|90|01|01|12|3456|7|\n  +--------------------+\n  |A|B |C |D |E | F  |G|\n  +-+--+--+--+--+----+-+\n  ```\n\n- A -> The century: A=2 From (1900-1999), A=3 From (2000-2099)\n- B~D (Date of birth):\n    B -> Year of birth\n    C -> Month of birth\n    D -> Day of birth\n- E -> Governorate code ex: {12: "Al Daqhlia"}\n- F -> Unique code. (Odd is male, Even is female)\n- G -> Check digit for verification\n\n## How to run tests\n\n- Lib test: `pytest -s tests/test_national_id.py`\n- Test the endpoint `python3 -m pytest -s tests/test_endpoint.py`\n\n## Dockerfile\n\n- You can build and run the dockerfile in `docker directory`\n  `docker build -t waleedhammam/national_id .`\n',
    'author': 'waleedhammam',
    'author_email': 'waleed.hammam@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/waleedhammam/national_id',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
